/* 
 * Please note that this class is a minimally modified version of the SerialPortHelper class developed by Hari.For more info 
 * about the origional class/code check out: http://forums.netduino.com/index.php?/topic/366-netduino-serial-port-code-review/
 */

using System;
using System.IO.Ports;
using System.Text;
using System.Threading;

using System.Collections;
using System.Collections.Generic;

namespace SPH_Windows
{
    public class WindowsSPH
    {
        #region Properties

        // The serial port
        public static System.IO.Ports.SerialPort serialPort;

        // Serial port buffer
        const int bufferMax = 1024;
        static byte[] buffer = new Byte[bufferMax];
        static int bufferLength = 0;

        // Default com-port
        static string defaultPort = "COM5";

        #endregion Properties

        #region SerialPort

        /// <summary>
        /// Open the serialport and set properties
        /// </summary>
        /// <param name="portName"></param>
        /// <param name="baudRate"></param>
        /// <param name="parity"></param>
        /// <param name="dataBits"></param>
        /// <param name="stopBits"></param>
        public WindowsSPH(string portName, int baudRate, Parity parity, int dataBits, StopBits stopBits)
        {
            serialPort = new SerialPort(portName, baudRate, parity, dataBits, stopBits);
            serialPort.ReadTimeout = 10; // Set to 10ms. Default is -1?!
            serialPort.DataReceived += new SerialDataReceivedEventHandler(serialPort_DataReceived);
            serialPort.Open();
        }

        /// <summary>
        /// Open the serialport with default properties
        /// </summary>
        public WindowsSPH()
        {
            serialPort = new SerialPort(defaultPort, 9600, Parity.None, 8, StopBits.One);
            serialPort.ReadTimeout = 10; // Set to 10ms. Default is -1?!
            serialPort.DataReceived += new SerialDataReceivedEventHandler(serialPort_DataReceived);
            serialPort.Open();
        }

        /// <summary>
        /// Close the serialport
        /// </summary>
        public void CloseSerialPort()
        {
            serialPort.Close();
        }

        #endregion SerialPort

        #region Read/Input

        /// <summary>
        /// Data recieved event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void serialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            lock (buffer)
            {
                int bytesReceived = serialPort.Read(buffer, bufferLength, bufferMax - bufferLength);
                if (bytesReceived > 0)
                {
                    bufferLength += bytesReceived;
                    if (bufferLength >= bufferMax)
                        throw new ApplicationException("Buffer Overflow.  Send shorter lines, or increase lineBufferMax.");
                }
            }
        }

        /// <summary>
        /// Perform serial input update read to user defined value
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        public string ReadTo(string value)
        {
            string line = string.Empty;

            lock (buffer)
            {
                //-- Look for Return char in buffer --
                for (int i = 0; i < bufferLength; i++)
                {
                    //-- Consider EITHER CR or LF as end of line, so if both were received it would register as an extra blank line. --
                    if (buffer[i] == Convert.ToByte(value))
                    {
                        buffer[i] = 0; // Turn NewLine into string terminator
                        line = "" + new string(Encoding.UTF8.GetChars(buffer)); // The "" ensures that if we end up copying zero characters, we'd end up with blank string instead of null string.
                        //Debug.Print("LINE: <" + line + ">");
                        bufferLength = bufferLength - i - 1;
                        Array.Copy(buffer, i + 1, buffer, 0, bufferLength); // Shift everything past NewLine to beginning of buffer
                        break;
                    }
                }
            }

            return line;
        }

        /// <summary>
        /// Perform serial input update read to CR and/or LR
        /// </summary>
        /// <returns></returns>
        public string ReadLine()
        {
            string line = "";

            lock (buffer)
            {
                //-- Look for Return char in buffer --
                for (int i = 0; i < bufferLength; i++)
                {
                    //-- Consider EITHER CR or LF as end of line, so if both were received it would register as an extra blank line. --
                    if (buffer[i] == '\r' || buffer[i] == '\n')
                    {
                        buffer[i] = 0; // Turn NewLine into string terminator
                        line = "" + new string(Encoding.UTF8.GetChars(buffer)); // The "" ensures that if we end up copying zero characters, we'd end up with blank string instead of null string.
                        //Debug.Print("LINE: <" + line + ">");
                        bufferLength = bufferLength - i - 1;
                        Array.Copy(buffer, i + 1, buffer, 0, bufferLength); // Shift everything past NewLine to beginning of buffer
                        break;
                    }
                }
            }

            return line;
        }

        #endregion Read/Input

        #region Print/Output

        /// <summary>
        /// Write data to port
        /// </summary>
        /// <param name="line"></param>
        public void Print(string line)
        {
            System.Text.UTF8Encoding encoder = new System.Text.UTF8Encoding();
            byte[] bytesToSend = encoder.GetBytes(line);
            serialPort.Write(bytesToSend, 0, bytesToSend.Length);
        }

        /// <summary>
        /// Write data (as line) and \r to port
        /// </summary>
        /// <param name="line"></param>
        public void PrintLine(string line)
        {
            Print(line + "\r");
        }

        #endregion Print/Output
    }
}
